//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      Filename                :               ConfigReader.h                          //
//                                                                                      //
//      Author                  :               Raman Dhillon                           //
//                                                                                      //
//      Version                 :               1.0                                     //
//                                                                                      //
//      Location                :               Pervasive Communication Lab             //
//                                              University of Colorado at Boulder       //
//                                                                                      //
//      Change Log                                                                      //
//      --------------------------------------------------------------------------      //
//      Change          Author          Date            Reason                          //
//                                                                                      //
//      --------------------------------------------------------------------------      //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////


#ifndef _CONFIG_READER_H
#define _CONFIG_READER_H

#include <iostream>

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/parsers/AbstractDOMParser.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMImplementationRegistry.hpp>
#include <xercesc/dom/DOMBuilder.hpp>
#include <xercesc/dom/DOMException.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMError.hpp>
#include <xercesc/dom/DOMLocator.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMAttr.hpp>

#include "DOMCount.h"
#include <string>
#include <stdlib.h>

#if defined(XERCES_NEW_IOSTREAMS)
#include <fstream>
#else
#include <fstream.h>
#endif

#include "TrafficInfo.h"


using namespace std;


class ConfigReader
{

  public:

	~ConfigReader();

	static ConfigReader* getConfigInstance(char *); //get the singleton instance

	int initializeXMLReader();

	int printXMLTree();

	TrafficInfo* buildTrafficScenario(char* scenarioName);

  protected:
	
	ConfigReader(){}
	ConfigReader(const char*);
	ConfigReader(const ConfigReader& );
	ConfigReader& operator=(const ConfigReader&);

  private:
	
	DOMShowErrorHandler* errorHandler;
    	DOMBuilder* parser ;
    	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc; 
	const char* xmlFile;   
 
	bool doNamespaces;
    	bool doSchema;
    	bool schemaFullChecking;
    	bool errorOccurred;

	static ConfigReader* pConfigInstance;  // singleton instance

	char* getNodeName(DOMNode* node);
	int printNode(DOMNode* node);
	DOMNode* getNode(char* commandName);

};

#endif
