//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      Filename                :               HTTPHandler.cpp                         //
//                                                                                      //
//      Author                  :               Raman Dhillon                           //
//                                                                                      //
//      Version                 :               1.0                                     //
//                                                                                      //
//      Location                :               Pervasive Communication Lab             //
//                                              University of Colorado at Boulder       //
//                                                                                      //
//      Change Log                                                                      //
//      --------------------------------------------------------------------------      //
//      Change          Author          Date            Reason                          //
//                                                                                      //
//      --------------------------------------------------------------------------      //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////


#include "HTTPHandler.h"

struct MemoryStruct {
  char *memory;
  size_t size;
};

  int HTTPHandler::sendHTTPGet(char* URL)
  {

	if (URL != NULL){
	
		curl = curl_easy_init();

		if(curl) {
    			curl_easy_setopt(curl, CURLOPT_URL, URL);
    			res = curl_easy_perform(curl);
			
			curl_easy_cleanup(curl);

			return 0;
		}
	}
	return -1;
  }

  int HTTPHandler::sendHTTPPost(char* URL, char* content)
  {

	curl = curl_easy_init();
    
	if(curl) {
	    	curl_easy_setopt(curl, CURLOPT_URL, URL);
    		curl_easy_setopt(curl, CURLOPT_POSTFIELDS, content);

    		// if we don't provide POSTFIELDSIZE, libcurl will strlen() by itself 
 
    		curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, strlen(content));

    		res = curl_easy_perform(curl);
 	
		curl_easy_cleanup(curl);
	
		return 0;
    } 

    return -1;

  }

  size_t HTTPHandler::write_data(void *ptr, size_t size, size_t nmemb,void *ourpointer)
  {
  
	size_t realsize = size * nmemb;
  	struct MemoryStruct *mem = (struct MemoryStruct *)ourpointer;
	int flag = 0; 

	if(mem->memory)
    		mem->memory = (char *)realloc(mem->memory, mem->size + realsize + 1);
  	else
	{
    		mem->memory = (char *)malloc(mem->size + realsize + 1);
	}

	if (mem->memory) {
    		memcpy(&(mem->memory[mem->size]), ptr, realsize);
    		mem->size += realsize;
    		mem->memory[mem->size] = 0;
  	}


	return realsize;
  } 

  int HTTPHandler::sendHTTPGetFile(char* URL)
  {
	struct MemoryStruct chunk;

  	chunk.memory=NULL; 
  	chunk.size = 0;   


	if (URL != NULL){

		curl = curl_easy_init();
	
		if(curl) {
    			curl_easy_setopt(curl, CURLOPT_URL, URL);
			curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);     		
			//curl_easy_setopt(curl, CURLOPT_VERBOSE, 1);

			curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *)&chunk);

  			curl_easy_setopt(curl, CURLOPT_USERAGENT, "libcurl-agent/1.0");

			res = curl_easy_perform(curl);

			//cout << ", File size=" << chunk.size / 1000 << " kB";

			curl_easy_cleanup(curl);
			free(chunk.memory);

			return chunk.size;
		}
	}
	return -1;
  }

  int HTTPHandler::sendHTTPPostFile(char* URL, char* content)
  {

    curl = curl_easy_init();
   
    if(curl) {
    	curl_easy_setopt(curl, CURLOPT_URL, URL);
    	curl_easy_setopt(curl, CURLOPT_POSTFIELDS, content);

    	// if we don't provide POSTFIELDSIZE, libcurl will strlen() by itself 
 
    	curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, strlen(content));

    	res = curl_easy_perform(curl);
 	
	curl_easy_cleanup(curl);

	return 0;
    } 

    return -1;

 }

