//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      Filename                :               LogNormalDist.h                         //
//                                                                                      //
//      Author                  :               Raman Dhillon                           //
//                                                                                      //
//      Version                 :               1.0                                     //
//                                                                                      //
//      Location                :               Pervasive Communication Lab             //
//                                              University of Colorado at Boulder       //
//                                                                                      //
//      Change Log                                                                      //
//      --------------------------------------------------------------------------      //
//      Change          Author          Date            Reason                          //
//                                                                                      //
//      --------------------------------------------------------------------------      //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#ifndef _LOGNORMAL_DIST_H
#define _LOGNORMAL_DIST_H

#include <iostream>
#include <math.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

using namespace std;

class LogNormalDist
{
  double mean, stdDev;
  double zeta, sigma;

  const gsl_rng_type *T;
  gsl_rng *r;

  public:

	LogNormalDist(double m, double s, int seed):mean(m), stdDev(s)
	{
 		gsl_rng_env_setup();
	
	        T = gsl_rng_default;
		gsl_rng_default_seed = seed;
	
                r = gsl_rng_alloc(T);

		sigma = sqrt(log(1 + pow((stdDev/mean),2)));
		zeta = log(mean) - 0.5 * sigma * sigma;
	}

	~LogNormalDist() 
	{
		gsl_rng_free(r);
	}

	void setParam(double m, double s)
	{
		mean = m;
		stdDev = s;
		sigma = sqrt(log(1 + pow((stdDev/mean),2)));
		zeta = log(mean) - 0.5 * sigma * sigma;
	}

	void showParam()
	{
		cout << "\nMean = " << mean;
		cout << "\nStandard Deviation = " << stdDev << endl;
	}

	double generateRandomVar()
	{
		double x = gsl_ran_lognormal(r, zeta, sigma);
		return x;
	}

};

#endif
