#include "POSIXThread.h"

POSIXThread::POSIXThread()
{
   currentState = THREAD_IDLE;
};


bool POSIXThread::operator==(const POSIXThread& obj) const
{
   return (pthread_equal(threadID, threadID) != 0);
};


bool POSIXThread::operator!=(const POSIXThread& obj) const
{
   return !operator==(obj);
};


void POSIXThread::run()
{
   setThreadState(THREAD_RUNNING);
   execute();
   setThreadState(THREAD_DEAD);
};



void* POSIXThread::threadFunction(void* pt)
{
   POSIXThread* thrd = reinterpret_cast<POSIXThread*>(pt);
   if (thrd)
   {
      thrd->run();
   }
   return NULL;
};


bool POSIXThread::start(bool thread_join)
{
   int result = pthread_create(&threadID,NULL,&(POSIXThread::threadFunction),this);

   if (result == 0)
   {
      if (!thread_join)
      {
         pthread_detach(threadID);
      }
      return true;
   }
   else
   {
      setThreadState(THREAD_DEAD);
      return false;
   }
};


int POSIXThread::self() const
{
   return (int)threadID;
}


void POSIXThread::detach(void) const
{
   // pthread_t tempThreadID = threadID;
   if (pthread_detach(threadID) != 0)
   {
	// error
   }
   // tempThreadID = 0; // this is done so as not to get an "unused-warning"
};


int POSIXThread::join(void)
{
   int status = 0;
   if (pthread_join(threadID, (void**)&status) != 0)
   {
	// error
   }
   return status;
};


bool POSIXThread::setPriority(int priority)
{

   if ((priority > 10) || (priority < 0)) return false;


   struct sched_param threadParameter;

   threadParameter.sched_priority = priority;

   if (pthread_setschedparam(threadID, SCHED_OTHER, &threadParameter) != 0)
   {
      return false;
   }
   else
   {
      return true;
   }

};


int POSIXThread::getPriority()
{
   struct sched_param threadParameter;
   int                policy = 0;

   if(pthread_getschedparam(threadID, &policy, &threadParameter) == 0)
   {
      return -1;
   }
   else
   {
      return threadParameter.sched_priority;
   }
};


void  POSIXThread::yield(void) const
{
   sched_yield();
};


void POSIXThread::terminate()
{

   if (pthread_cancel(threadID) != 0)
   {
	// error
   }
};


void POSIXThread::exit()
{
   pthread_exit((void**)0);
};


bool POSIXThread::setThreadState(int state)
{
   currentState = state;
   return true;
};


int POSIXThread::getThreadState()
{
   int state = currentState;
   return state;
};

