//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      Filename                :               ParetoDist.h                            //
//                                                                                      //
//      Author                  :               Raman Dhillon                           //
//                                                                                      //
//      Version                 :               1.0                                     //
//                                                                                      //
//      Location                :               Pervasive Communication Lab             //
//                                              University of Colorado at Boulder       //
//                                                                                      //
//      Change Log                                                                      //
//      --------------------------------------------------------------------------      //
//      Change          Author          Date            Reason                          //
//                                                                                      //
//      --------------------------------------------------------------------------      //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////


#ifndef _PARETO_DIST_H
#define _PARETO_DIST_H

#include <iostream>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

using namespace std;

#define MEAN 0
#define MIN 1


class ParetoDist
{

  double paretoExp, mean, min;
  const gsl_rng_type *T;
  gsl_rng *r;  

  public:

 	ParetoDist(double a, double m, int type, int seed) : paretoExp(a)
	{
		if(type == MEAN)
		{
		  mean = m;
		  min = mean * (paretoExp - 1) / paretoExp;
		
		}
		else
		{
		  min = m;
		  mean = paretoExp * min/ (paretoExp - 1) ;
		}
	
		gsl_rng_env_setup();
		
		T = gsl_rng_default;
		gsl_rng_default_seed = seed;

		r = gsl_rng_alloc(T);

	}
	~ParetoDist()
	{
		gsl_rng_free(r);
	}

	void setParam(double a, double m, int type)
	{

		paretoExp = a;

		if(type == MEAN)
		{
		  mean = m;
		  min = mean * (paretoExp - 1) / paretoExp;
		
		}
		else
		{
		  min = m;
		  mean = paretoExp * min/ (paretoExp - 1) ;
		}

	}

	void showParam()
	{
		cout << "\nPareto exponent, a = " << paretoExp;
		cout << "\nMean value, m = " << mean;
		cout << "\nMinimum value, Xm = " << min << endl;
	}	

	double generateRandomVar()
	{
		double x = gsl_ran_pareto(r, paretoExp, min);
		return x;
	}

};

#endif
